/*
    memfile.h

    Copyright (c) 1996, by Global Constructs.  All Rights Reserved.

    last modified 12/30/96
*/
#ifdef __cplusplus
extern "C" {
#endif

#ifdef NOMEMFILE_H

/* Error values returned in errmem by memory file functions */
#define badFileHandle		EBADF
#define fileModeConflict	EINVAL
#define accessModeConflict	EINVAL
#define loadFailFileExists	EEXIST
#define exclCreateFailed	EEXIST
#define exclAccessDenied	EACCES
#define fileNotReadable		EACCES
#define fileNotWritable		EACCES
#define stackFileDup		EACCES
#define noTruncOnRead		EACCES
#define fileDoesNotExist	ENOENT
#define syncDiskFileOnly	EACCES
#define outOfMemory			ENOSPC
#define invalidBufferPtr	EINVAL
#define maxFileSizeExceeded	ENOSPC
#define stackFileSeek		EACCES
#define seekPastBeginning	EINVAL
#define invalidParameter	EINVAL
#define stackFileRead		EACCES
#define pullNotAllowed		EACCES
#define syncSeekFailure		EINVAL
#define syncWriteFailure	EIO
#define tooManyOpenHandles	EMFILE
#define loadFailModeConflict	EACCES

/* file access definitions */
#define I_RDONLY		O_RDONLY
#define I_WRONLY		O_WRONLY
#define I_RDWR			O_RDWR
#define I_APPEND		O_APPEND
#define I_CREAT			O_CREAT
#define I_TRUNC			O_TRUNC
#define I_BINARY		O_BINARY
#define I_EXCL			O_EXCL
#define I_LOAD			0

/* file mode definitions */
#define M_IREAD			S_IREAD
#define M_IWRITE		S_IWRITE
#define M_IFIFO			0
#define M_IFILO			0
#define M_ISYNC			0

#define mclose			close
#define mcreat(b,c)		open(b, O_WRONLY | O_CREAT | O_TRUNC, c)
#define mdup			dup
#define meof			eof
#define mheapsize(a)	;
#define minfo(q)		NULL
#define mopen			open
#define mpeek			read		/* does not handle file position reset */
#define mpull			read		/* does not handle file position */
#define mpush			write		/* does not write at end of file */
#define mread			read
#define mseek			lseek
#define msetsize(b,c)	0
#define mstrerror		strerror
#define mtell(a)		lseek(a, 0, SEEK_CUR)
#define mtruncate(d,e)	ftruncate(d,e)
#define mwrite			write

#define errmem			errno

#elif !defined(MEMFILE_H)

#define MEMFILE_H

#ifndef COMMON_H
typedef signed long			int32;
typedef unsigned long		int32u;
typedef signed short		int16;
typedef unsigned short		int16u;

#include <sys/stat.h>
#else
#include STAT_H
#endif

/*
	Error values returned in errmem by memory file functions
*/
typedef enum {
	badFileHandle = 1,
	fileModeConflict,	/* more than one file type is set */
	accessModeConflict,	/* >= 1 file access (I_RDONLY,I_WRONLY,I_RDWR) set */
	loadFailFileExists,	/* disk file failed to load due to existing memory file */
	exclCreateFailed,	/* file exists and cannot be created exclusive */
	exclAccessDenied,	/* file already open with other file handle */
	fileNotReadable,	/* file mode or access does not permit reading */
	fileNotWritable,	/* file mode or access does not permit writing */
	stackFileDup,		/* stacked memory files are allowed one file handle only */
	stackFileSeek,		/* stacked files may get current position only */
	stackFileRead,		/* read not permitted on stacked file, use pull instead */
	pullNotAllowed,		/* pull not permitted on normal file, use read instead */
	noTruncOnRead,		/* file truncation is not permitted on a read only file */
	fileDoesNotExist,	/* file to be opened does not exist */
	syncDiskFileOnly,	/* sync on opening of non-disk file attempted */
	syncSeekFailure,	/* seek on disk based sync file failed on synchronization */
	syncWriteFailure,	/* write on disk based sync file failed to synchronize */
	outOfMemory,		/* insufficient memory to perform operation */
	diskReadError,		/* error occurred while loading disk based file */
	invalidBufferPtr,	/* NULL pointer provided on mread, mwrite, mpush, or mpull */
	maxFileSizeExceeded,/* maximum file size limitation exceeded */
	seekPastBeginning,	/* seek attempted to before beginning of the file */
	invalidParameter,	/* invalid parameter value set */
	tooManyOpenHandles	/* all available file handles are assigned */
} MemoryError;

/* file access definitions */
#define I_RDONLY		0x0001
#define I_WRONLY		0x0002
#define I_RDWR			0x0004
#define I_APPEND		0x0008
#define I_CREAT			0x0010
#define I_TRUNC			0x0020
#define I_BINARY		0x0040
#define I_EXCL			0x0080
#define I_LOAD			0x0100	/* load file from disk on open */

/* file mode definitions */
#define M_IREAD			000001
#define M_IWRITE		000002
#define M_INORM			000004
#define M_IFIFO			000010
#define M_IFILO			000020
#define M_ISYNC			000040

/*
	This structure provide detailed information about a particular file.
*/
typedef struct _fileinfo {
	int32 filesize;			/* size of file in bytes */
	int32 maxfilesize;		/* maximum file size to write */
	int32 totalclusters;	/* total memory reserved for file, whether used or not */
	int32u filemode;		/* type of memory file */
	int16u syncrate;		/* sync rate for the memory file */
	int16u synccount;		/* number of writes to perform before synchronization */
	int16 numsegments;		/* number of memory segments for file */
} fileInfo;

extern int16 errmem;

#define mcreat(b,c)		mopen(b, O_WRONLY | O_CREAT | O_TRUNC, c)
#define mpush			mwrite
#define mtell(a)		mseek(a, 0, SEEK_CUR)

/*
    int16 mclose (int16 handle)

    Closes memory file and releases all memory.  It will close any related disk
    based file which is synchronized.

    Parameter
    handle  File handle number.

    Returns zero if successful.  Otherwise, it returns -1 and  errmem is set to
    indicate the error.

    Errors
    badFileHandle
    syncSeekFailure
    syncWriteFailure

    KBLs    Closure of the last file handle for the memory file removes it from memory
            losing all information.
*/
int16 mclose( int16 );
/*
	int16 mdup (int16 handle)

	Duplicates file handle given as parameter and returns new file handle which
	also points to the memory file.

	Parameter
	handle	File handle number.

	Returns the new file handle, or -1 on an error and errmem is set.

	Errors
	badFileHandle
	stackFileDup
	exclAccessDenied
	outOfMemory
	tooManyOpenHandles

	KBLs	None.
*/
int16 mdup( int16 );
/*
    int16 meof (int16 handle)

    Tests for end of file for memory file.

    Parameter
    handle  File handle number.

    Returns 1 if at end of file, 0 if not at end of file, and -1 on an error
    and errmem is set.

    Error
    badFileHandle

    KBLs    None.
*/
int16 meof( int16 );
/*
    void mheapsize (int32 heapsize)

    Changes the default file allocation size.  To reset the heap size to the
    default, set 'heapsize' to 0L.  Heap size requests of less than zero set
    the heap size to the default.

    Parameter
    heapsize    Size in bytes to set allocation to, four byte boundary aligned.

    Void return.
    KBLs    The function will set the heap size to the next available four byte
            boundary size if a non-four byte boundary value is provided.
*/
void mheapsize( int32 );
/*
	fileInfo *minfo (int16 handle)

	Gets detailed information on a particular memory file.  This routine
	allocates space for this information so be sure to call free(fileInfo *)
	when finished.

	Parameter
	handle	File handle number.

	Returns a pointer to a fileInfo structure, or NULL on an error with errmem set.

	Errors
	badFileHandle
	outOfMemory

	KBLs	None.
*/
fileInfo *minfo( int16 );
/*
	int mopen (const char *path, int16 access, int16u mode)

	Opens a memory file for read, write, append, and create access.

	Parameters
	path	A NULL pointer may be passed for the filename, doing so will
			create a file with no name.  Multiple uses of this method will
			create multiple files, not multiple file handles to the same file.

	access	The access mode is established by a combination of the bits defined
			as follows.
			
			I_RDONLY	Access the memory file for read access only
			I_WRONLY	Access the memory file for write access only
			I_RDWR		Access the memory file for read and write access
			I_APPEND	Records will be written at the end of the memory file
			I_CREAT		Create the memory file if it does not already exist
			I_TRUNC		Truncate the memory file to contain no data if the file
						already exists
			I_BINARY	Open the file in binary mode, which does not change the
						data on a read/write or push/pull.  This is the default
						access mode whether or not it has been specified.
			I_EXCL		Open a memory file for exclusive access.  If the file
						exists and then the open will fail (since there is a
						file handle assigned to that file already).
			I_LOAD		Create a memory file and load a disk based file into
						it.  If the memory file exists then the open will fail.
						I_CREAT is set by default when I_LOAD is set, whether
						or not I_CREAT has been specified.

	mode	The file mode permissions for the file are specified by an optional
			argument as a combination of bits as follows.  The mode parameter
			is ignored unless the access mode bit I_CREAT or I_LOAD is set and
			the file does not exist.

			M_IREAD		Permit read access to the created memory file
			M_IWRITE	Permit write access to the created memory file
			M_IFIFO		Set file type as a stacked first-in/first-out memory file
			M_IFILO		Set file type as a stacked first-in/last-out memory file

	A stacked file (FIFO or FILO) may be opened with one I_RDWR file handle or
	one I_RDONLY and I_WRONLY file handle only.  Note that multiple stacked
	memory files may be open at one time.

	Note that use of a NULL pointer in the filename parameter is not POSIX
	compliant if you intend on changing the memory file code to POSIX code.
	File types of FIFO and FILO will produce file types normal for your
	system if NOMEMFILE_H is defined.

    Returns a handle for the file.  When an error occurs while opening the file,
    -1 is returned and the variable errmem is set.  A disk file loaded as a
	memory file can return a file handle and set errmem.  This can occur in such
	cases as a partial read or disk I/O error.

	Errors
	fileModeConflict
	accessModeConflict
	loadFailFileExists
	exclCreateFailed
	exclAccessDenied
	fileNotReadable
	fileNotWritable
	stackFileDup
	noTruncOnRead
	fileDoesNotExist
	syncDiskFileOnly
	outOfMemory
	diskReadError
	tooManyOpenHandles

    KBLs    'path' is stored as a filename without regard for current paths or
				other conventions.
            Wildcards and special characters are considered valid characters in
				filename.
            All files are handled as binary only.
			Shared memory files are not supported.
*/
int16 mopen( const char *, int16, int16u );
/*
    int32 mpeek (int16 handle, void *buffer, int32u len)

    Reads a memory block from the beginning of the stack for FIFO operations,
    the end of the stack for FILO operations, or the current file position for
    normal files changing the file position or size.

    Parameters
    handle  File handle number
    buffer  Pointer to input buffer
    len     Length of record to read into the buffer

    Returns the number of bytes read from the file.  When there is no error,
    this is the number given by the len argument.  When the end of the file is
    encountered before the read completes, the return value will be less than
    the number of bytes requested.  A value of -1 may be returned in the case
    of some input errors.  When an error has occurred, errmem contains a value
    indicating the type of error that has been detected.

    Errors
    badFileHandle
    invalidBufferPtr
    fileNotReadable

    KBLs    As mopen().
*/
int32 mpeek( int16, void *, int32u );
/*
	int32 mpull (int16 handle, void *buffer, int32u len)

	Reads a memory block from the beginning of the stack for FIFO operations
	or the end of the stack for FILO operations, reducing the size of the
	stack file in the process.

	Parameters
	handle	File handle number
	buffer	Pointer to input buffer
	len		Length of record to read into the buffer

	Returns the number of bytes read from the file.  When there is no error,
	this is the number given by the len argument.  When the end of the file is
	encountered before the read completes, the return value will be less than
	the number of bytes requested.  A value of -1 may be returned in the case
	of some input errors.  When an error has occurred, errmem contains a value
	indicating the type of error that has been detected.

	Errors
	badFileHandle
	invalidBufferPtr
	pullNotAllowed
	fileNotReadable
	syncSeekFailure
	syncWriteFailure

	KBLs	As mopen().
*/
int32 mpull( int16, void *, int32u );
/*
	int32 mread (int16 handle, void *buffer, int32u len)

	Reads a memory block from the current file position.

	Parameters
	handle	File handle number.
	buffer	Pointer to buffer to read record into.
	len		Length of record to read.

	Returns the number of bytes read from the file.  When there is no error,
	this is the number given by the len argument.  When the end of the file is
	encountered before the read completes, the return value will be less than
	the number of bytes requested.  A value of -1 may be returned in the case
	of some input errors.  When an error has occurred, errmem contains a value
	indicating the type of error that has been detected.

	Errors
	badFileHandle
	invalidBufferPtr
	stackFileRead

	KBLs	As mopen().
*/
int32 mread( int16, void *, int32u );
/*
	int32 mseek (int16 handle, int32 offset, int16 whence)

	Sets file position of memory file.

	Parameters
	handle	File handle numbers.
	offset	Value to shift file position depending upon whence value.
	whence	Position from which to start offset.  Valid values are

			SEEK_SET	Offset from start of memory file
			SEEK_CUR	Offset from current file position
			SEEK_END	Offset from end of memory file

	Returns file position if successful.  Otherwise, it returns -1 and errmem is
	set to indicate the error.

	Errors
	badFileHandle
	invalidParameter
	stackFileSeek
	seekPastBeginning

	KBLs	For FIFO and FILO file types, mseek(fh,0,SEEK_CUR) or mtell(fh)
			are only allowed.
*/
int32 mseek( int16, int32, int16 );
/*
	int16 msetsize(int16 handle, int32 maxsize)

	Sets the maximum size a file may reach.  mwrite() and mpush() will not exceed
	this amount if set, although they can write to an existing file which exceeds
	this limit if the write does not extend the file.  mread(), mseek(), and
	mpull() are not affected by this limit in any way.

	Parameters
	handle	The file handle on which to set a file size limit.
	maxsize	The maximum file size in bytes to which the file may be extended.
			Setting maxsize to zero will disable the file size limit for that
			file handle.

	Returns zero if successful, otherwise -1 on an error and errmem is set.

	Errors
	badFileHandle
	invalidParameter

	KBLs	None.
*/
int16 msetsize( int16, int32 );
/*
	int16 mstat (const char *path, struct stat *buf)

	Provides statistical information on a memory file.  For more detailed
	information used the routine mfileinfo().

	Parameters
	path	Path and file name of memory file on which to get statistics.
	buf		Pointer to valid memory buffer.

	Returns zero when the information is successfully obtained, -1 on an error
	with errmem set.

	Errors
	fileDoesNotExist
	invalidBufferPtr

	KBLs	This function will only get the file size and the hard links used
			by the file.
*/
int16 mstat( const char *, struct stat * );
/*
    const char *mstrerror( int16 errnum )

    mstrerror maps the error number contained in errmem to an error message
    string.

    Parameter
    errmem  The number of the error usually returned by errmem.

    Returns a pointer to a string containing the requested error message.  The
    pointer is to a constant string and should not be modified by the calling
    routine.

    KBLs    None.

*/
const char *mstrerror( int16 );
/*
    int16 msync ( int16 handle, int16u num )

    Set the number of write operations to occur between each synchronization
    with the corresponding disk file.

    Parameters
    handle  File handle number
    num     Number of write operations between synchronizations.  The default
            is 100, setting num to zero will disable synchronization, clear
            the M_ISYNC mode flag for the file, and close the disk based file.

    Returns zero if successful, otherwise -1 and errmem is set

    badFileHandle

    KBLs    None.
*/
int16 msync( int16 , int16u );
/*
    int16 mtruncate (int16 handle, int32 newsize)

    Truncates file to new size if the file was larger than than newsize

    Parameters
    handle  File handle number
    newsize Size to change file to if larger

    Returns zero if successful, otherwise -1 and errmem is set.

    Errors
    badFileHandle
    invalidParameter
    fileNotWritable
    syncSeekFailure
    syncWriteFailure

    KBLs    None.
*/
int16 mtruncate( int16, int32 );
/*
    int mwrite(int handle, void *buffer, unsigned len)

	Writes a memory block at the current file position.

	Parameters
	handle	file handle number
	buffer	pointer to data buffer of outputted information
	len		length of block to write from the buffer

    Returns the number of bytes written to the file.  When there is no error,
	this is the number given by the len argument.  In the case of an error, such
	as there being no space available to contain the file data, the return value
	will be less than the number of bytes transmitted.  A value of -1 may be
	returned in the case of some output errors.  When an error has occurred,
	errmem contains a value indicating the type of error that has been detected.

	Errors
	badFileHandle
	invalidBufferPtr
	fileNotWritable
	maxFileSizeExceeded
	syncSeekFailure
	syncWriteFailure
	outOfMemory

    KBLs    As mopen().  If M_ISYNC option is used and an error occurs in the
			'\0' byte fill for a file position past the end of the file, then
			the sync counter will not be decremented for that one memory file
			write operation.
*/
int32 mwrite( int16, void *, int32u );

#endif  /* end of memfile.h */

#ifdef __cplusplus
};
#endif

