/*
    memlib.h

    Copyright (c) 1996, 1997, by Global Constructs.  All Rights Reserved.

    last modified 10/17/97
*/

#ifndef MEMLIB_H

#define MEMLIB_H
#include "common.h"
#include "memfile.h"

#define DEFAULT_MHEAP   0x2000L /* default heap size 8192 bytes */

/*
    This structure provides necessary information on a memory segment basis for a
    particular file handle.  The following features apply:

    1.  Free space is only available in the last memory segment for the file.
        a.  The pointer 'm_next' is NULL for only the last segment in the list.
        b.  All other segments will have a free space equal to zero.
    2.  The memory segment size never changes once a segment block is created.
    3.  The block size will equal the memory segment size for a full memory block.
    4.  A memory segment is not necessarily the same size as another memory segment.
*/
typedef struct _memlist {
	/*
		The next pointer 'm_next' points to the next memory structure in the
		list.  If there is no next structure (at the end of the list) then it
		points to NULL.  The previous pointer 'm_prev' points to the previous
		memory structure in the list.  If there is no previous structure (at
		the beginning of the list) then it points to the last structure in the
		list.

		m_prev is never NULL.  When there is only one structure in the list
		then m_prev equals fptr->memsegments;
	*/
    struct _memlist *m_next, *m_prev;
    char *m_ptr;
    int32 m_size;      /* size of memory segment */
    int32 fblock_size; /* amount of block that is used by file */
    int32 free_size;   /* always equal to m_size - fblock_size */
} memList;

/*
	This structure provides necessary information on a file-by-file basis.
*/
typedef struct _filelist {
	struct _filelist *f_next;
	char *filename;			/* pointer to null terminated filename */
	memList *memsegments;	/* memory segment list for file */
	int32u rbuf[4];			/* file's reference buffer */
	int32 filesize;			/* size of file in bytes */
	int32 clustersize;		/* total memory reserved for file, whether or not used */
	int32 startpos;			/* start position for FIFO file */
	int16u filemode;		/* type of memory file */
	int16u syncrate;		/* sync rate for the memory file */
	int16u synczero;		/* number of writes to perform before synchronization */
	int16 numsegments;		/* number of memory segments for file */
	int32 synchandle;		/* file handle to disk based file */
} fileList;

/*
	This structure provides specific file handling information.
*/
typedef struct _filehandle {
	struct _filehandle *h_next; /* next file handle for memory file */
	fileList *file;		/* Pointer to file attached to file handle */
	struct {
		memList *fpptr;	/* Pointer to last used memory list structure */
		int32 blockpos;	/* Block position within block pointed to by fpptr */
		int32 filepos;	/* Last file position set by read/write/seek */
	} last;
	int32 filepos;
	int32 maxfilesize;	/* Maximum file size to write */
	int16 filehandle;
	int16 fileaccess;
	int16 eof;			/* boolean TRUE if at EOF, else FALSE */
} fileHandle;

#define firstlist	gc1stlist
extern fileList *firstlist;
#define firsthandle	gc1sthandle
extern fileHandle *firsthandle;	
extern int32 gcmemheap;

/*
    fileList *gcgffp (const char *name)

    Gets the pointer to the specified file using the file name

    Parameter
    name    path and file name of memory file

    Returns pointer to filelist or NULL if not found

    KBLs    None.
*/
#define getfilefrompath	gcgffp
fileList *gcgffp( const char * );
/*
	int gcsyfi (fileList *fptr)

	Synchronizes memory file to disk by copying over the disk file with the
	memory file.

	Parameter
	fptr	Pointer to valid file list

	Errors
	syncSeekFailure
	syncWriteFailure

	Returns zero on success, otherwise -1 and errmem is set.

	KBLs	None.
*/
#define syncfile	gcsyfi
int gcsyfi( fileList * );
/*
	fileHandle *gcgefh (int16 handle)

	Get pointer to file handle structure

	Parameter
	handle	File handle number

	Returns pointer to file handle structure, or NULL if not found.

	KBLs	None.
*/
#define getfilehandle	gcgefh
fileHandle *gcgefh( int16 );
/*
	int gcadfh(fileList *fptr, int access)

	Adds a file handle structure for a particular file

	Parameters
	fptr	Pointer to file list
	access	File access mode

	Returns the file handle, otherwise -1 on an error and errmem is set.

	Errors
	outOfMemory
	tooManyOpenHandles

	KBLs	None.
*/
#define addfilehandle	gcadfh
int16 gcadfh( fileList * , int16 );
/*
	memList *gcadms (fileList *fptr)

	Adds memory segment and configures it for use

	Parameter
	fptr	Pointer to file list

	Returns a pointer to the memory segment otherwise NULL if an error occurs.
	An error value will be set in errmem.

	Error
	outOfMemory

	KBLs	None.
*/
#define addmemorysegment	gcadms
memList *gcadms( fileList * );
/*
	memList *getsegmentpositionread (const fileHandle *hptr, int32 *pos)
	memList *getsegmentpositionwrite (const fileHandle *hptr, int32 *pos)

	Returns a pointer to the current memory segment based upon the current
	file position, and taking into account a FIFO start position.

	Parameters
	hptr	Pointer to file handle list
	pos		Pointer to block adjusted memory position

	Returns the current memory list pointer for the current file position,
	otherwise NULL in the event of zero bytes available in any memory segment
	of the file.

	KBLs	None.
*/
#define getsegmentpositionread(a,b)		gcgesp(a,b,TRUE)	/* read, push, peek */
#define getsegmentpositionwrite(a,b)	gcgesp(a,b,FALSE)
memList *gcgesp( const fileHandle * , int32 *, int );

#endif  /* end of memlib.h */

