/*
    memperf.c

    Copyright (c) 1996, 1997, by Global Constructs. All Rights Reserved.

    This source file includes a program demonstrating the speed of the
    memlib.lib static library.  The program creates and compares a
    file of 10,000 1K byte records entirely in memory or disk within a
    temporary directory.  The writes will occur at the beginning of the
    file while the reads the next record: next record for M_INORM, last
	record for M_IFILO, and first record for M_IFIFO.

    Arguments [-(tm?)] [filename]
    -t          Put file in temporary directory
    -mf         Put file in memory FIFO file
	-ml			Put file in memory FILO file
	-mn			Put file in random access memory file
	-a			Put file in memory array
    -?          Display help window
    -h          Display help window
    filename    Provide filename and possibly path also
    default     Default settings '-mn filename.out'

    last modified 1/18/97
*/
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <string.h>
#ifndef __BORLANDC__
#	include <unistd.h>
#endif
#if defined(__WATCOMC__)
#	include <direct.h>
#elif defined(__BORLANDC__)
#	include <dir.h>
#endif
#include "memfile.h"

#define RECORDSIZE	1024
#define RECORDNUMS	10000

#define DISKFILE    0
#define MEMFILE     1
#define ARRAYFILE	2

#define MEMNORM		0x1
#define MEMFIFO		0x2
#define MEMFILO		0x4

extern int errno;

void main(int argc, const char *argv[])
{
    char *fn = "filename.out", *path;
    char buffer[RECORDSIZE] = "Indigo Windigo Winabego Bagel Nigel Nile Anile", *buf=NULL;
    char input[RECORDSIZE];
    int i, ret, loctype = MEMFILE, filetype = MEMNORM;
    clock_t times[4];
    int fh=-1;
	long int arraycnt = 0;

	mheapsize(1024 * 1024);
    for (i=1;i<argc;i++)
        switch(argv[i][0]) {
            case '-':
                switch(argv[i][1]) {
                    case 't': case 'T':
                        loctype = DISKFILE;
                        path = getenv("temp");
                        if (path) {
                            ret = chdir(path);
                            if (ret < 0) {
                                printf("Error: %s\n", strerror(errno));
                                exit(1);
                            }
                            printf("Temporary path = %s\n", path);
                        }
						filetype = 0;
                        break;
                    case 'm': case 'M':
                        loctype = MEMFILE;
						switch(argv[i][2]) {
							case 'f': case 'F':
								filetype = MEMFIFO;
								break;
							case 'l': case 'L':
								filetype = MEMFILO;
								break;
							case 'n': case 'N':
								filetype = MEMNORM;
								break;
							default:
								printf("Error: Unknown option -m%c\n", argv[i][2]);
								exit(1);
								break;
						}
                        break;
					case 'a': case 'A':
						loctype = ARRAYFILE;
						filetype = 0;
						break;
                    case '?':
                    case 'h':
                        printf("Copyright (c) 1996, 1997, by Global Constructs.  All Rights Reserved.\n\n");
                        printf("Performance program of memory file system.\n");
                        printf("\tVersion 1.00\n\n\n");
                        printf("memperf [-(mt?)] [fn]\n");
						printf("\t-a\tCreate array of records.\n");
                        printf("\t-mf\tCreate FIFO memory file.\n");
                        printf("\t-ml\tCreate FILO memory file.\n");
                        printf("\t-mn\tCreate NORM memory file.\n");
                        printf("\t-t\tCreate file in temporary directory.\n");
                        printf("\t-?\tDisplay this help window\n");
                        printf("\t-h\tDisplay this help window\n");
                        printf("\t[fn]\tProvide path/filename for file\n\n");
                        exit(0);
                        break;
                    default:
                        printf("Error: Unknown option '%c'\n", argv[i][1]);
                        exit(1);
                        break;
                }
                break;
            default:
                fn = argv[i];
                break;
        }
    /*
        Set up disk based file as a random access file.  Set up a memory file
        as FILO.
    */
	switch(filetype) {
		case MEMFIFO:
			strcpy(input, " FIFO");
			break;
		case MEMFILO:
			strcpy(input, " FILO");
			break;
		case MEMNORM:
			strcpy(input, " NORM");
			break;
		default:
			input[0] = '\0';
			break;
	}
    printf("Creating a%s%s.\n", input, (loctype == MEMFILE ? " memory file" : (loctype == ARRAYFILE ? "n array" : " temporary file")));
    switch(loctype) {
        case DISKFILE:
            times[0] = clock();
            fh = open(fn, O_RDWR | O_CREAT | O_EXCL, S_IREAD | S_IWRITE);
            if (fh < 0) {
                printf("open Error: %s\n", strerror(errno));
                exit(1);
            }
            times[0] = clock() - times[0];
            break;
        case MEMFILE:
            times[0] = clock();
			switch(filetype) {
				case MEMFIFO:
         		   fh = mopen(fn, I_RDWR | I_CREAT | I_EXCL, M_IREAD | M_IWRITE | M_IFIFO);
					break;
				case MEMFILO:
        		    fh = mopen(fn, I_RDWR | I_CREAT | I_EXCL, M_IREAD | M_IWRITE | M_IFILO);
					break;
				case MEMNORM:
        		    fh = mopen(fn, I_RDWR | I_CREAT | I_EXCL, M_IREAD | M_IWRITE);
					break;
			}
            if (fh < 0) {
                printf("open Error: %s\n", mstrerror(errmem));
                exit(1);
            }
            times[0] = clock() - times[0];
            break;
		case ARRAYFILE:
            times[0] = clock();
			buf = calloc(RECORDNUMS, RECORDSIZE);
			if (buf == NULL) {
				printf("allocation Error: Could not allocate memory for array.\n");
				exit(1);
			}
            times[0] = clock() - times[0];
			break;
    }
    printf("Writing 10,000 records to file.\n");
    switch(loctype) {
        case DISKFILE:
            times[1] = clock();
            for (i=0;i<RECORDNUMS;i++)
                if (write(fh, buffer, sizeof(buffer)) < sizeof(buffer)) {
                    printf("write Error: %s\n", strerror(errno));
                    exit(1);
                }
            times[1] = clock() - times[1];
            break;
        case MEMFILE:
            times[1] = clock();
            for (i=0;i<RECORDNUMS;i++)
				/* mpush() is just a macro for mwrite() */
                if (mwrite(fh, buffer, sizeof(buffer)) < sizeof(buffer)) {
                    printf("write Error: %s\n", mstrerror(errmem));
                    exit(1);
                }
            times[1] = clock() - times[1];
            break;
		case ARRAYFILE:
            times[1] = clock();
            for (i=0;i<RECORDNUMS;i++) {
				strcpy(&buf[arraycnt], buffer);
				arraycnt += RECORDSIZE;
			}
            times[1] = clock() - times[1];
			break;
    }
    printf("Reading 10,000 records in%s order.\n", input);
    switch(loctype) {
        case DISKFILE:
            times[2] = clock();
            for (i=RECORDNUMS-1;i>=0;i--) {
                ret = lseek(fh, i * sizeof(buffer), SEEK_SET);
                if (ret < 0) {
                    printf("seek Error: %s\n", strerror(errno));
                    exit(1);
                }
                ret = read(fh, input, sizeof(buffer));
                if (ret < 0) {
                    printf("read Error: %s\n", strerror(errno));
                    exit(1);
                }
            }
            times[2] = clock() - times[2];
            break;
        case MEMFILE:
            times[2] = clock();
			switch(filetype) {
				case MEMFILO: case MEMFIFO:
					for (i=RECORDNUMS-1;i>=0;i--) {
						ret = mpull(fh, input, sizeof(buffer));
						if (ret < 0) {
							printf("pull Error: %s\n", mstrerror(errmem));
							exit(1);
						}
					}
					break;
				case MEMNORM:
					ret = mseek(fh, 0, SEEK_SET);
					if (ret < 0) {
						printf("seek Error: %s\n", mstrerror(errmem));
						exit(1);
					}
					for (i=0;i<RECORDNUMS;i++) {
						ret = mread(fh, input, sizeof(buffer));
						if (ret < 0) {
							printf("read Error: %s\n", mstrerror(errmem));
							exit(1);
						}
					}
					break;
			}
            times[2] = clock() - times[2];
            break;
		case ARRAYFILE:
            times[2] = clock();
			arraycnt -= RECORDSIZE;
            for (i=RECORDNUMS-1;i>=0;i--) {
				strcpy(input, &buf[arraycnt]);
				arraycnt -= RECORDSIZE;
			}
            times[2] = clock() - times[2];
			break;
    }
    printf("Closing file.\n");
    switch(loctype) {
        case DISKFILE:
            times[3] = clock();
            close(fh);
            times[3] = clock() - times[3];
			remove(fn);
            break;
        case MEMFILE:
            times[3] = clock();
            mclose(fh);
            times[3] = clock() - times[3];
            break;
		case ARRAYFILE:
            times[3] = clock();
			free(buf);
            times[3] = clock() - times[3];
			break;
    }
    printf("\nReported results.\n\n");
    printf("Create file   = %4.3f secs\n", (float) times[0] / CLOCKS_PER_SEC);
    printf("Write records = %4.3f secs\n", (float) times[1] / CLOCKS_PER_SEC);
    printf("Read records  = %4.3f secs\n", (float) times[2] / CLOCKS_PER_SEC);
    printf("Close file    = %4.3f secs\n", (float) times[3] / CLOCKS_PER_SEC);
	printf("input buffer  = %s\n", input);
}

